package jdk.internal.org.objectweb.asm.commons;

import jdk.internal.org.objectweb.asm.signature.SignatureVisitor;
import jdk.internal.org.objectweb.asm.signature.SignatureWriter;
import jdk.internal.org.objectweb.asm.signature.SignatureReader;
import jdk.internal.org.objectweb.asm.Handle;
import jdk.internal.org.objectweb.asm.Type;

public abstract class Remapper
{
    public Remapper() {
    }
    
    public String mapDesc(String typeDescriptor) {
        Type type = Type.getType(typeDescriptor);
        switch (type.getSort()) {
            case 9: {
                String str = this.mapDesc(type.getElementType().getDescriptor());
                for (int i = 0; i < type.getDimensions(); ++i) {
                    str = 91 + str;
                }
                return str;
            }
            case 10: {
                String map = this.map(type.getInternalName());
                if (map != null) {
                    return 76 + map + 59;
                }
                break;
            }
        }
        return typeDescriptor;
    }
    
    private Type mapType(Type type) {
        switch (type.getSort()) {
            case 9: {
                String s = this.mapDesc(type.getElementType().getDescriptor());
                for (int i = 0; i < type.getDimensions(); ++i) {
                    s = 91 + s;
                }
                return Type.getType(s);
            }
            case 10: {
                String map = this.map(type.getInternalName());
                return (map != null) ? Type.getObjectType(map) : type;
            }
            case 11: {
                return Type.getMethodType(this.mapMethodDesc(type.getDescriptor()));
            }
            default: {
                return type;
            }
        }
    }
    
    public String mapType(String internalName) {
        if (internalName == null) {
            return null;
        }
        return this.mapType(Type.getObjectType(internalName)).getInternalName();
    }
    
    public String[] mapTypes(String[] array) {
        String[] array2 = null;
        boolean n = false;
        for (int i = 0; i < array.length; ++i) {
            String s = array[i];
            String map = this.map(s);
            if (map != null && array2 == null) {
                array2 = new String[array.length];
                if (i > 0) {
                    System.arraycopy(array, 0, array2, 0, i);
                }
                n = true;
            }
            if (!(n == 0)) {
                array2[i] = ((map == null) ? s : map);
            }
        }
        return n ? array2 : array;
    }
    
    public String mapMethodDesc(String methodDescriptor) {
        if ("()V".equals(methodDescriptor)) {
            return methodDescriptor;
        }
        Type[] argumentTypes = Type.getArgumentTypes(methodDescriptor);
        StringBuilder sb = new StringBuilder("(");
        for (int i = 0; i < argumentTypes.length; ++i) {
            sb.append(this.mapDesc(argumentTypes[i].getDescriptor()));
        }
        Type returnType = Type.getReturnType(methodDescriptor);
        if (returnType == Type.VOID_TYPE) {
            sb.append(")V");
            return sb.toString();
        }
        sb.append(')').append(this.mapDesc(returnType.getDescriptor()));
        return sb.toString();
    }
    
    public Object mapValue(Object o) {
        if ((o instanceof Type)) {
            return this.mapType((Type)o);
        }
        if ((o instanceof Handle)) {
            Handle handle = (Handle)o;
            return new Handle(handle.getTag(), this.mapType(handle.getOwner()), this.mapMethodName(handle.getOwner(), handle.getName(), handle.getDesc()), this.mapMethodDesc(handle.getDesc()));
        }
        return o;
    }
    
    public String mapSignature(String signature, boolean b) {
        if (signature == null) {
            return null;
        }
        SignatureReader signatureReader = new SignatureReader(signature);
        SignatureWriter signatureWriter = new SignatureWriter();
        SignatureVisitor remappingSignatureAdapter = this.createRemappingSignatureAdapter(signatureWriter);
        if (b) {
            signatureReader.acceptType(remappingSignatureAdapter);
        }
        else {
            signatureReader.accept(remappingSignatureAdapter);
        }
        return signatureWriter.toString();
    }
    
    protected SignatureVisitor createRemappingSignatureAdapter(SignatureVisitor signatureVisitor) {
        return new RemappingSignatureAdapter(signatureVisitor, this);
    }
    
    public String mapMethodName(String s, String s2, String s3) {
        return s2;
    }
    
    public String mapInvokeDynamicMethodName(String s, String s2) {
        return s;
    }
    
    public String mapFieldName(String s, String s2, String s3) {
        return s2;
    }
    
    public String map(String s) {
        return s;
    }
}
